/**********************************************************/
/** Done by  |      Date     |  version |    comment     **/
/**------------------------------------------------------**/
/**   CEV    |    05-2016    |   1.0    |  creation/SDL1 **/
/**********************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <SDL.h>
#include "CEV_gifDeflate.h"
#include "CEV_gifToSurface.h"
#include "CEV_gif.h"
#include "rwtypes.h"



/**direct gif animation load from gif file*/
CEV_GifAnim * CEV_gifAnimLoad(const char* filename)
{/* TODO (cedric#1#): check errors / fitness */

    CEV_GifAnim *result = NULL;

    result = CEV_gifAnimLoadRW(SDL_RWFromFile(filename, "rb"), 1);

    if (!result)
        fprintf(stderr, "Err : SDL_GIFAnimLoad / Unable to extract animation from file\n");

    return result;
}


/**gif animation load from SDL_RWops */
CEV_GifAnim * CEV_gifAnimLoadRW(SDL_RWops* rwops, char freeSrc)
{/* TODO (cedric#1#): check errors / fitness */

    int i;
    uint8_t     *pixels     = NULL;
    L_GifFile   *gif        = NULL;
    SDL_Surface *tempSurf   = NULL,
                *main       = SDL_GetVideoSurface();
    CEV_GifAnim *anim       = NULL;

    /**perform compatibiliy control**/

    if ((main->format->BytesPerPixel != 4) || (main->format->BitsPerPixel != 32))
    {
        fprintf(stderr,"Sorry, main surface is incompatible with gif functions.\n");
        return NULL;
    }

    anim = malloc(sizeof(CEV_GifAnim));
    if (!anim)
        goto err_exit;

    readWriteErr = 0;

    /*extracting datas from gif file*/
    gif = L_gifLoadRW(rwops);

    if(!gif)
        goto err_1;

    /*CREATING MAIN SURFACE AKA the user one*/
    tempSurf = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCCOLORKEY,
                                        gif->lsd.width,
                                        gif->lsd.height,
                                        32,
                                        main->format->Rmask,
                                        main->format->Gmask,
                                        main->format->Bmask,
                                        main->format->Amask);

    if(!tempSurf)
    {/*on error*/
        fprintf(stderr,"Err / SDL_GIFAnimLoad : %s\n", SDL_GetError());
        goto err_2;
    }
    else
    {/*filling informations*/
        main = SDL_DisplayFormatAlpha(tempSurf);

        if(main!=NULL)
        {
            SDL_FreeSurface(tempSurf);
            anim->display.surface  = main;
            anim->display.pos.x    = 0;
            anim->display.pos.y    = 0;
            anim->display.pos.h    = gif->lsd.height;
            anim->display.pos.w    = gif->lsd.width;

            gif->colorShift.rShift   = main->format->Rshift;
            gif->colorShift.gShift   = main->format->Gshift;
            gif->colorShift.bShift   = main->format->Bshift;
            gif->colorShift.aShift   = main->format->Ashift;
        }
        else
        {
          fprintf(stderr,"Err / SDL_GIFAnimLoad : %s\n", SDL_GetError());
          goto err_2;
        }
    }

    /**Starting to create every pic, one by one*/

    if (L_gifAnimInit(anim, gif))
        goto err_3;

    for (i=0; i<anim->status.imgNum; i++)
    {
        pixels = malloc(gif->image[i].descriptor.width * gif->image[i].descriptor.height * sizeof(uint32_t));

        if(pixels)
        {
            switch (gif->image[i].descriptor.imgPack.interlace)
            {
                case 0:
                    L_gifFillSurface(pixels, gif, i);
                break;

                case 1:
                    L_gifFillSurfaceInterlace(pixels, gif, i);
                break;

                default:
                break;
            }
        }
        else
        {
            fprintf(stderr,"Err / SDL_GIFAnimLoad : %s\n", strerror(errno));
            goto err_3;
        }

        /*filling informations from gif file into animation*/
        anim->pictures[i].pixels     = pixels;
        anim->pictures[i].pos.x      = gif->image[i].descriptor.leftPos;
        anim->pictures[i].pos.y      = gif->image[i].descriptor.topPos;
        anim->pictures[i].pos.h      = gif->image[i].descriptor.height;
        anim->pictures[i].pos.w      = gif->image[i].descriptor.width;
        anim->pictures[i].dispMethod = gif->image[i].control.packField.disposalMethod;
        anim->pictures[i].time       = gif->image[i].control.delayTime*10;

        /*get sure frame rects fit into surface rect*/
        L_gifFitBoxInto(&anim->pictures[i].pos, &anim->display.pos);

    }

    L_gifFileFree(gif); /*free raw gif datas*/

    /*be ready to display first logical frame in case of no call to SDL_GIFAnimAuto*/
    L_gifBlit(anim);


    if(readWriteErr)/*well.. wait and see if it's a problem*/
        fprintf(stderr,"Info / SDL_GIFAnimLoad : some R/W err has occured, file may be unstable\n");

    if (freeSrc)
        SDL_RWclose(rwops);

    return anim;

/**error management from here**/
err_3 :
    CEV_gifAnimFree(anim);
    anim = NULL; /*security for err_1*/


err_2 :
    L_gifFileFree(gif);

err_1 :
    free(anim);

err_exit :
    return NULL;
}


/**returns embedded comment*/
char *CEV_gifComment(CEV_GifAnim *anim)
{
    return anim->status.comment;
}


/**returns embedded version*/
char *CEV_gifVersion(CEV_GifAnim *anim)
{
    return anim->status.version;
}


/**returns embedded signature*/
char *CEV_gifSignature(CEV_GifAnim *anim)
{
    return anim->status.signature;
}


/**returns User's texture**/
SDL_Surface *CEV_gifSurface(CEV_GifAnim *anim)
{
    return anim->display.surface;
}


/**returns num of pics*/
int CEV_gifFrameNum(CEV_GifAnim *anim)
{
    return anim->status.imgNum;
}


/**forces next frame*/
void CEV_gifFrameNext(CEV_GifAnim *anim)
{
    L_gifBlit(anim);
}


/**sets animation speed*/
void CEV_gifSpeedSet(CEV_GifAnim *anim, int num, int speed)
{
    int i;

    if ((num >= anim->status.imgNum) || num < GIF_ALL)
        return;/*nothing to be done*/

    else if (num >=0 )
        anim->pictures[num].time = (uint16_t)speed;/*the num one only*/

    else/*all frames*/
        for(i=0; i<anim->status.imgNum; i++)
            anim->pictures[i].time = (uint16_t)speed;
}

/**sets loop reading mode*/
void CEV_gifLoopMode(CEV_GifAnim *anim, unsigned int loopMode)
{
    if(loopMode > GIF_STOP)
        return;/*nothing to be done*/

    anim->status.loopMode = loopMode;

    switch (loopMode)
    {
        case GIF_REPEAT_REV :/*modes starting from last frame*/
        case GIF_ONCE_REV :
            anim->status.imgAct = anim->status.imgNum-1;
        break;

        default:/*other modes*/
            anim->status.imgAct = 0;
        break;
    }

    anim->status.refresh = 1;               /*force refreshing*/
    anim->status.time    = SDL_GetTicks();  /*restart from now*/
}


/**resets loop animation**/
void CEV_gifLoopReset(CEV_GifAnim *anim)
{
    switch (anim->status.loopMode)
    {
        case GIF_REPEAT_FOR :
        case GIF_ONCE_FOR :
        case GIF_FORTH_BACK:
            anim->status.imgAct = 0;
            anim->status.loopDone = 0;
        break;

        case GIF_REPEAT_REV :
        case GIF_ONCE_REV :
            anim->status.imgAct = anim->status.imgNum-1;
            anim->status.loopDone = 0;
        break;

        default:
        break;
    }

    anim->status.refresh = 1;               /*force refreshing*/
    anim->status.time    = SDL_GetTicks();  /*restart from now*/
}


/**updates animations**/
char CEV_gifAnimAuto(CEV_GifAnim *anim)
{
    char sts = 0;

    unsigned int actTime,
                 now = SDL_GetTicks();

    int imgAct = anim->status.imgAct;

    if(!anim)
        return 0;

    /*if first call*/
    if(!anim->status.time)
        anim->status.time = now;

    /*since last time ?*/
    actTime = now - anim->status.time;

    if((actTime >= anim->pictures[imgAct].time) || anim->status.refresh)
    {/*it's time or it's forced by else function*/
        sts = L_gifBlit(anim);
        anim->status.time = now;
    }

    return sts;
}


/**reverses play mode*/
void CEV_gifReverse(CEV_GifAnim *anim)
{
    switch (anim->status.loopMode)
    {
        case GIF_REPEAT_FOR :
             anim->status.loopMode = GIF_REPEAT_REV;
        break;

        case GIF_REPEAT_REV :
             anim->status.loopMode = GIF_REPEAT_FOR;
        break;

        case GIF_ONCE_FOR :
            anim->status.loopMode = GIF_ONCE_REV;
        break;

        case GIF_ONCE_REV :
            anim->status.loopMode = GIF_ONCE_FOR;
        break;

        case GIF_FORTH_BACK :
            anim->status.direction ^= 1;
        break;

        default:
        break;
    }
}


/**frees animation structure**/
void CEV_gifAnimFree(CEV_GifAnim *anim)
{
    int i;

    if(!anim)
        return;

    for(i=0; i<anim->status.imgNum; i++)
        free(anim->pictures[i].pixels); /*every pixels table*/

    free(anim->pictures);               /*the frames table*/
    free(anim->status.comment);         /*comments if some*/
    SDL_FreeSurface(anim->display.surface);/*main surface*/
    free(anim);                         /*and itself*/
}


/**returns if loop is playing or not**/
char CEV_gifLoopStatus(CEV_GifAnim* anim)
{
    return !(anim->status.loopDone || (anim->status.loopMode==GIF_STOP));
}


/**returns used method**/
char CEV_gifMethod(CEV_GifAnim* anim, uint8_t num)
{
    if(num>anim->status.imgNum-1)
        num = 0;

    return (char)anim->pictures[num].dispMethod;
}


/**sets method**/
void SDL_gifMethodSet(CEV_GifAnim* anim, int num, uint8_t method)
{
    int i;

    if(num > anim->status.imgNum-1 || num < GIF_ALL)
        return;

    else if (num==GIF_ALL)
    {
        for(i=0; i<anim->status.imgNum; i++)
            anim->pictures[i].dispMethod = method;
    }
    else
        anim->pictures[num].dispMethod = method;
}
